open Structure_lexer
open Cli


let version_number = "0.7"


 

(*
  Tool: namealign
  Author: Oliver Bandel

   namealign,
    the cool tool to conveniontly rename files automatically.


  ======================================================
  Urheber/Copyright => Oliver Bandel
  No warranty provided: use at your own risk .
  You can redistribute this software under GPL 2.0
  (see file "COPYING").
  ======================================================


  $Revision: 1.10 $

*)



type record_t = { path: string;
                  basename: string;
                  structure: Structure_lexer.splitted_t;
                  macro_str: Structure_lexer.macro_t;
                  newname: string option }




(* looks unconvenient, but submodule is used during development time
   to prevent pollution of the namespace by opening Inix-module.
   Can definitely made much cuter... *)
module Test =
(
  struct
    open Unix
    let is_file f = match (handle_unix_error stat f).st_kind with S_REG -> true | _ -> false
    let is_dir  d = match (handle_unix_error stat d).st_kind with S_DIR -> true | _ -> false
  end
   :
  sig
    val is_file: string -> bool
    val is_dir : string -> bool
  end
)

 
  


(* has much unused code, that can be thrown away in later versions *)
module Hash =
(
 struct
  include Hashtbl
  let keys tbl =
    let keytable = Hashtbl.create 10000 in
    let keylist = ref [] in
      Hashtbl.iter ( fun k v -> Hashtbl.replace keytable k 1 ) tbl;
      Hashtbl.iter ( fun k v -> keylist := k :: !keylist ) keytable;
      !keylist

    
  let append tbl x y =
    let value =
      try
        find tbl x
      with Not_found -> ref [y]
     in
       value := y::!value;
       Hashtbl.replace tbl x value
    



  let replace_max_parts tbl x y =
    let value =
      try
        find tbl x
      with Not_found -> y
     in
       let newvalue = List.map2 (fun x1 x2 -> if x1.plen > x2.plen then x1 else x2 ) y value
       in
         Hashtbl.replace tbl x newvalue

 end
  :
 sig
    type ('a, 'b) t = ('a, 'b) Hashtbl.t
    val create : int -> ('a, 'b) t
    val find : ('a, 'b) t -> 'a -> 'b
    val find_all : ('a, 'b) t -> 'a -> 'b list
    val add : ('a, 'b) t -> 'a -> 'b -> unit
    val replace : ('a, 'b) t -> 'a -> 'b -> unit
    val replace_max_parts : ('a, Structure_lexer.part_t list) t -> 'a -> Structure_lexer.part_t list -> unit
    val append : ('a, 'b list ref) t -> 'a -> 'b -> unit
    val iter : ('a -> 'b -> unit) -> ('a, 'b) t -> unit
    val keys : ('a, 'b) t -> 'a list
 end
)



let recs    = Hash.create 1000 (* records of the files *)
let strmax  = Hash.create 1000 (* maximum-layout-structures goes here *)




let scan_string string =
  let result = ref [] in
    try
      let lexbuf = Lexing.from_string string in
      while true do
        result := (Structure_lexer.normal_lexer lexbuf) :: !result
      done; !result
    with Structure_lexer.Eof -> List.rev !result
        (* hier knnte man auch den MININT-rewrite einbauen! *)


let get_strtype x = match x.ptype with
       Integer -> String.make x.plen 'i'
     | Letter  -> String.make x.plen 'l'
     | Blank   -> String.make x.plen 'b'
     | Dot     -> String.make x.plen 'd'
     | Slash   -> String.make x.plen 's'
     | Other   -> String.make x.plen 'o'





(** This function creates a record *)
let scan2record fullname =
   let path = Filename.dirname fullname
   and basename = Filename.basename fullname
   in
     let res = (scan_string basename) in
     let ms = List.map (fun el -> el.ptype) res in
      { path = path; basename = basename; structure = res; macro_str = ms; newname = None }



let read_strings_from_stdin () =
  let r_stringlist = ref [] in
    try
      while true do
        r_stringlist := read_line() ::!r_stringlist
      done; []
    with End_of_file -> List.rev !r_stringlist


let read_lines_from_file fname =
  let lines = ref [] in
    let ichan = open_in fname in
    try
      while true
      do
        lines := (input_line ichan) :: !lines
      done; !lines
    with End_of_file -> close_in ichan; List.rev !lines


exception Error_in_match
exception Data_error



let align_to_maxsize part maxpart =
  let nsize = max maxpart.plen part.plen in (* should be size *)
  match part.ptype with
      Blank  -> if options.singlerep then "_" else String.make nsize '_'                            (* all Blanks will be substituted *)
   | Slash   -> if options.singlerep then "_" else String.make nsize '_'                            (* all slashes substituted *)
   | Letter  -> if options.letter = true
                then part.ptext ^ String.make (nsize - part.plen) 'z' (* missing letters added *) 
                else part.ptext                                       (* unchanged string *)
   | Integer -> String.make (nsize - part.plen) '0' ^ part.ptext (* leading 0's added *)
   | Dot     -> if options.singlerep then "." else String.make nsize '.'                            (* missing dots added *)
   | Other   -> if options.singlerep then "_" else String.make nsize '_' (* all others substituted *)

 
let align_structure splitted splitted_max = List.map2 align_to_maxsize splitted splitted_max



(** parsing the command line. *)
let parse () = (*print_endline "OK, starting the parse!";*)
    Arg.parse [
         ("-letter",   Arg.Unit (fun () -> options.letter <- true ),     "       letter will also be changed" );
         ("-na",       Arg.Unit (fun () -> options.no_action <- true ),  "           no action: show only what would be done if this flag is NOT set" );
         ("-ad",       Arg.Unit (fun () -> options.allow_dirs <- true ),  "           allow dirs: allow directories also to be renamed!" );
         ("-minint",   Arg.Unit (fun () -> options.minint <- true ),     "       integers: minimal size (remove leading 0's)" );
         ("-s",   Arg.Unit (fun () -> options.singlerep <- true ),     "            single repitition: one replacement char for any NON-ALNUM parts in the name)" );
         ("-show_all", Arg.Unit (fun () -> options.show_all <- true ),   "     show all: even files that will not be renamed will be shown" );
         ("-verbose",  Arg.Unit (fun () -> options.show_all <- true ),   "      verbose: same as show_all: even files that will not be renamed will be shown" );
         ("-version",  Arg.Unit (fun () -> options.version <- true ),    "      show program version and exit" )
       (* => DEFAULT (hardcoded) !!!
       *)
              ] 
         ( fun name -> ()  ) 
         "Use the \"namealign\" tool with following options:"




let rename_file oldname newname =
  try Unix.rename oldname newname with
    Unix.Unix_error (code,s1,s2) -> Printf.fprintf Pervasives.stderr "%s * %s * %s\n" (Unix.error_message code)  s1  s2





let _ =

  parse(); (* Command Line Interface parsing *)
  if options.version then begin Printf.printf "namealign, version %s\n" version_number; exit 0  end;
  let namelist = read_strings_from_stdin () in

(* Wenn allow-dirs und Dir => die Files auch! *)
(* => Erzeugung des Filters *)

  (* next thing we do: filter for refular files (as well as links to files) *)
  let filelist = List.filter Test.is_file namelist in


  let reclist = List.rev_map scan2record filelist in (* records erzeugen *)


  (* The records now will be analyzed *)
  List.iter ( fun x -> Hash.add recs x.macro_str x; (* fge weiteren Eintrag zu *)
                       Hash.replace_max_parts strmax x.macro_str x.structure   (* Maxima der Microstr pro Macrostr *)
            ) reclist;


            (* fr alle Structures ... *)
  Hash.iter ( fun k v -> let maximum_layout = Hash.find strmax k in
                     (* show_structure v; print_endline "--"; show_structure maximum_layout; *)
                     let orig = v.basename (*String.concat "" (List.map (fun x -> x.ptext) v.structure) *) in
                     let aligned = String.concat "" (align_structure v.structure maximum_layout) in
                     let orig_fullname = String.concat "/" [v.path; orig] in
                     let aligned_fullname = String.concat "/" [v.path; aligned] in
        match ((orig <> aligned), (Sys.file_exists aligned_fullname), options.show_all, options.no_action) with
          | (false, _, true, _)      -> Printf.fprintf stdout "needs no renaming: \"%s\"\n" orig_fullname
          | (false, _, false, _)     -> () (* not necessary to print something *)
          | (true,true,true,_)          -> Printf.fprintf stdout "will not rename  : \"%s\" to \"%s\", because I would have to overwrite file!\n" orig_fullname aligned_fullname
          | (true,true,false,_)          -> Printf.fprintf stderr "will not rename  : \"%s\" to \"%s\", because I would have to overwrite file!\n" orig_fullname aligned_fullname
          | (true,false,true,true)   -> Printf.printf "possible renaming: \"%s\" -> \"%s\"\n" orig_fullname aligned_fullname;
          | (true,false,true,false)  -> Printf.printf "renaming: \"%s\" -> \"%s\"\n" orig_fullname aligned_fullname;
                                          rename_file orig_fullname aligned_fullname (* RENAMING *)
          | (true,false,false,true)  -> () (* nothing to do or print here! *)
          | (true,false,false,false) -> rename_file orig_fullname aligned_fullname (* silently do your work *) (* RENAMING *)

            ) recs





