{

(*
  Tool: namealign
  Author: Oliver Bandel

  ======================================================
  Urheber/Copyright => Oliver Bandel
  No warranty provided: use at your own risk .
  You can redistribute this software under GPL 2.0
  (see file "COPYING").
  ======================================================
*)

open Cli

type classify_t = Blank | Letter | Integer | Dot | Slash | Other
type macro_t = classify_t list
type part_t = { ptext: string; ptype: classify_t; plen : int }
type splitted_t = part_t list

exception Eof
exception Unclassified

}

rule normal_lexer = parse
   | ['\n' '\r' ' ']+    { let s = Lexing.lexeme lexbuf in {ptext = s; ptype =Blank; plen = String.length s} }
   | ['a'-'z' 'A'-'Z']+  { let s = Lexing.lexeme lexbuf in {ptext = s;  ptype = Letter; plen = String.length s} }
   | ['' '' '' '' '' '' '']+  { let s = Lexing.lexeme lexbuf in {ptext = s; ptype = Letter; plen = String.length s} }
   | ['0'-'9']+          { let lex = (Lexing.lexeme lexbuf) in let s = if options.minint then Helpers.rlz lex else lex in
      {ptext = s; ptype = Integer; plen = String.length s} }
   | '.'+                { let s = Lexing.lexeme lexbuf in {ptext = s;  ptype = Dot; plen = String.length s} }
   | '/'+                { let s = Lexing.lexeme lexbuf in {ptext = s;  ptype = Slash; plen = String.length s} }
   (*
   | '-'+                { let s = Lexing.lexeme lexbuf in {ptext = s;  ptype = Minus; plen = String.length s } }
   *)
   | [^'\n' '\r' ' ' 'a'-'z' 'A'-'Z' '' '' '' '' '' '' '' '0'-'9' '.' '/' ]+ { let s = Lexing.lexeme lexbuf in {ptext = s; ptype = Other; plen = String.length s} }
   | _  { raise Unclassified } (* should never be possible -> only a hard programming error could do that *)
   | eof { raise Eof }


{

}
